
__author__ = 'Lukas Leufen'
__date__ = '2020-09-21'

import inspect
from typing import Union, Dict

from mlair.helpers import remove_items


class AbstractDataHandler:

    _requirements = []

    def __init__(self, *args, **kwargs):
        pass

    @classmethod
    def build(cls, *args, **kwargs):
        """Return initialised class."""
        return cls(*args, **kwargs)

    @classmethod
    def requirements(cls):
        """Return requirements and own arguments without duplicates."""
        return list(set(cls._requirements + cls.own_args()))

    @classmethod
    def own_args(cls, *args):
        """Return all arguments (including kwonlyargs)."""
        arg_spec = inspect.getfullargspec(cls)
        list_of_args = arg_spec.args + arg_spec.kwonlyargs
        return remove_items(list_of_args, ["self"] + list(args))

    @classmethod
    def transformation(cls, *args, **kwargs):
        return None

    def apply_transformation(self, data, inverse=False, **kwargs):
        """
        This method must return transformed data. The flag inverse can be used to trigger either transformation or its
        inverse method.
        """
        raise NotImplementedError

    def get_X(self, upsampling=False, as_numpy=False):
        raise NotImplementedError

    def get_Y(self, upsampling=False, as_numpy=False):
        raise NotImplementedError

    def get_data(self, upsampling=False, as_numpy=False):
        return self.get_X(upsampling, as_numpy), self.get_Y(upsampling, as_numpy)

    def get_coordinates(self) -> Union[None, Dict]:
        """Return coordinates as dictionary with keys `lon` and `lat`."""
        return None
